var filtersTable;

// Create filters table
function initFiltersTable() {
    filtersTable = $("#filtersTable").DataTable({
        info: false,
        paging: false,
        searching: false,
        autoWidth: false,
        order: [],
        select: {
            style: "multi+shift",
            selector: 'th:first-child input[type="checkbox"]'
        },
        columnDefs: [
            { orderable: false, targets: "no-sort" },
            {
                targets: 0,
                checkboxes: { selectRow: true }
            }
        ],
        columns: [
            {
                // Select row checkbox column
                title: '<input type="checkbox" class="form-check-input table-select-all" />',
                data: null,
                orderable: false,
                className: "text-center col-1",
                render: function() {
                    return '<input type="checkbox" class="form-check-input table-select-row" />'
                }
            },
            { title: "ID", data: "id", visible: false },
            {
                title: "Name",
                data: "name",
                render: function(data, type, row) {
                    return `<a href="#" onclick="showEditFilterModal(${row.id})" class="text-decoration-none">${data}</a>`
                }
            },
            {
                title: "Keywords",
                data: "keywords",
                render: function(data, type) {
                    if (!data) return "-";
                    return data;
                }
            },
            {
                title: "Regex",
                data: "regex",
                render: function(data, type) {
                    if (!data) return "-";
                    return data;
                }
            },
            {
                title: "Type",
                data: "whitelist",
                render: function(data) {
                    return data ? "whitelist" : "blacklist";
                }
            }
        ]
    });
}

$("#filterAdvancedMode").on("change", function() {
    const advancedMode = $(this).prop("checked");
    setFilterAdvancedMode(advancedMode);
});

function setFilterAdvancedMode(advancedMode) {
    if (advancedMode) {
        $("#filterForm .simple-filtering").hide();
        $("#filterForm .advanced-filtering").show();
        $("#filterForm .advanced-filtering").val("");
    }
    else {
        $("#filterForm .advanced-filtering").hide();
        $("#filterForm .simple-filtering").show();
        $("#filterForm .simple-filtering").val("");
    }
}

$("#addFilterBtn").on("click", async function() {
    await showEditFilterModal(-1);
})

async function showEditFilterModal(filterId) {

    if (filterId === -1) {
        $("#filterFormModal input, #filterFormModal textarea").val("");
        $("#filterFormModal .form-create").show();
        $("#filterFormModal .form-edit").hide();
        $("#filterAdvancedMode").prop("checked", false);
        $("#filterAdvancedMode").trigger("change");
        $("#filterWhitelist").prop("checked", false);
    }
    else {
        const filter = filtersTable.row(function(idx, data, node) {
            return data.id === filterId;
        }).data();

        $("#filterName").val(filter.name);
        $("#filterKeywords").val(filter.keywords);
        $("#filterRegex").val(filter.regex);

        $("#filterAdvancedMode").prop("checked", filter.regex !== "");
        $("#filterAdvancedMode").trigger("change");

        $("#filterWhitelist").prop("checked", filter.whitelist);
        $("#filterWhitelist").trigger("change");

        $("#filterFormModal .form-create").hide();
        $("#filterFormModal .form-edit").show();
    }

    $("#filterId").val(filterId);
    $("#filterFormModal").modal("show");
}

$("#filterForm").on("submit", async function(event) {
    event.preventDefault();

    var advancedFiltering = $("#filterAdvancedMode").prop("checked");
        id = $("#filterId").val();
        name = $("#filterName").val();
        keywords = advancedFiltering ? "" : $("#filterKeywords").val();
        regex = advancedFiltering ? $("#filterRegex").val() : "";
        whitelistMode = $("#filterWhitelist").prop("checked");
        guildId = getCurrentlyActiveServer().guild_id;

    if (advancedFiltering) {
        keywords = "";
    }
    else {
        regex = ""
    }

    var filterPrimaryKey = await saveFilter(id, name, keywords, regex, whitelistMode, guildId);

    if (filterPrimaryKey) {
        showToast("success", "Filter Saved", "Filter ID " + filterPrimaryKey);
        await loadFilters(guildId);
        await loadFilterOptions(guildId);
    }

    $("#filterFormModal").modal("hide");
});

async function saveFilter(id, name, keywords, regex, whitelistMode, guildId) {
    var formData = new FormData();
    formData.append("name", name);
    formData.append("keywords", keywords);
    formData.append("regex", regex);
    formData.append("whitelist", whitelistMode)
    formData.append("guild_id", guildId);

    var response;

    try {
        if (id === "-1") response = await newFilter(formData);
        else response = await editFilter(id, formData);
    }
    catch (err) {
        showToast("danger", "Filter Error", err.responseText, 18000);
        return false
    }

    return response.id;
}

function clearExistingFilterRows() {
    $("#filtersTable thead .table-select-all").prop("checked", false).prop("indeterminate", false)
    filtersTable.clear().draw(false)
}

$("#refreshFilterBtn").on("click", async function() {
    loadFilters(getCurrentlyActiveServer().guild_id);
});

async function loadFilters(guildId) {

    if (!guildId)
        return;

    $("#deleteSelectedFiltersBtn").prop("disabled", true);
    clearExistingFilterRows();

    try {
        const filters = await getFilters(guildId);
        filtersTable.rows.add(filters.results).draw(false);
        $("#filtersTable thead .table-select-all").prop("disabled", filters.results.length === 0);
    }
    catch (err) {
        console.error(JSON.stringify(err, null, 4));
        showToast("danger", `Error Loading Filters: HTTP ${err.status}`, err.responseJSON.message, 15000);
    }
}

$(document).on("selectedServerChange", async function() {
    const activeServer = getCurrentlyActiveServer();
    await loadFilters(activeServer.guild_id);
});

$("#deleteSelectedFiltersBtn").on("click", async function() {

    var rows = filtersTable.rows(".selected").data();
    $.each(rows, async function() {
        await deleteFilter(this.id);
        showToast("danger", "Deleted Filter", "Filter ID: " + this.id)
    })

    setTimeout(async () => {
        const guildId = getCurrentlyActiveServer().guild_id;
        await loadFilters(guildId);
        loadFilterOptions(guildId);
    }, 500)
})

$("#deleteEditFilter").on("click", async function() {
    const filterId = $("#filterId").val();
    await deleteFilter(filterId);
    await loadFilters(getCurrentlyActiveServer().guild_id);
    $("#filterFormModal").modal("hide");
    showToast("danger", "Deleted Filter", "Filter ID: " + filterId);
});